/*
 * source/data/Screen.c
 * SpacePigs VectorMania
 * 
 * Screen and draw window
 */
#include <exec/types.h>
#include <intuition/intuition.h>

#include "vector/vector.h"
#include "graphics/graphics.h"
#include "edit/edit.h"

extern char ScreenTitle[];

struct NewScreen NewScreen =
{
    0, 0,			/* screen XY origin relative to View */
    640, 256,			/* screen width and height */
    3,				/* screen depth (number of bitplanes) */
    0, 1,			/* detail and block pens */
    HIRES,			/* display modes for this screen */
    CUSTOMSCREEN,		/* screen type */
    NULL,			/* pointer to default screen font */
    ScreenTitle,		/* screen title */
    NULL,			/* first in list of custom screen gadgets */
    NULL			/* pointer to custom BitMap structure */
};

UWORD Palette[] =
{
    0x0000,			/* color #0 BLACK       */
    0x0FFF,			/* color #1 WHITE       */
    0x0DDD,			/* color #2 GRAY1       */
    0x0BBB,			/* color #3 GRAY2       */
    0x0999,			/* color #4 GRAY3       */
    0x0777,			/* color #5 GRAY4       */
    0x0555,			/* color #6 GRAY5       */
    0x0333,			/* color #7 GRAY6       */
    0x00FF,			/* color #8             */
    0x0000,			/* color #9             */
    0x0F00,			/* color #10 RED        */
    0x00F0,			/* color #11 GREEN      */
    0x0FF0,			/* color #12 YELLOW     */
    0x000F,			/* color #13 BLUE       */
    0x0444,			/* color #14            */
    0x0FFF			/* color #15            */
#define PaletteColorCount 16
};

UWORD pens[] =
{
    2,				/* DETAILPEN */
    0,				/* BLOCKPEN */
    1,				/* TEXTPEN */
    1,				/* SHINEPEN */
    5,				/* SHADOWPEN */
    2,				/* FILLPEN */
    0,				/* FILLTEXTPEN */
    0,				/* BACKGROUNDPEN */
    10,				/* HIGHLIGHTTEXTPEN */
    ~0
};

struct TagItem NewScreenTags[] =
{
    {SA_Width, STDSCREENWIDTH},
    {SA_Height, STDSCREENHEIGHT},
    {SA_Depth, 3},
    {SA_DetailPen, 0},
    {SA_BlockPen, 1},
    {SA_Title, &ScreenTitle},
    {SA_SysFont, 1},
    {SA_Type, CUSTOMSCREEN},
    {SA_ShowTitle, TRUE},
    {SA_Pens, &pens},
    {SA_FullPalette, TRUE},
    {TAG_END}
};

#define DrawWinIDCMP	(IDCMP_NEWSIZE|IDCMP_MENUPICK|IDCMP_RAWKEY|IDCMP_ACTIVEWINDOW|IDCMP_INACTIVEWINDOW|IDCMP_INTUITICKS)

struct NewWindow NewDrawWindow =
{
    0, 0,			/* window XY origin relative to TopLeft of screen */
    640, 256,			/* window width and height */
    0, 3,			/* detail and block pens */
    DrawWinIDCMP,		/* IDCMP flags */
    BORDERLESS + ACTIVATE + NOCAREREFRESH | BACKDROP,	/* other window flags */
    NULL,			/* first gadget in gadget list */
    NULL,			/* custom CHECKMARK imagery */
    NULL,			/* window title */
    NULL,			/* custom screen pointer */
    NULL,			/* custom bitmap */
    -1, -1,			/* minimum width and height */
    -1, -1,			/* maximum width and height */
    CUSTOMSCREEN		/* destination screen type */
};

#define PALETTE Palette

struct TagItem NewDrawWindowTags[] =
{
    {WA_CustomScreen, NULL},	/* MSTE ifyllas fre ppna */
    {WA_Left, 0},
    {WA_Top, 0},
    {WA_Width, ~0},
    {WA_Height, ~0},
    {WA_AutoAdjust, TRUE},
    {WA_IDCMP, DrawWinIDCMP},
    {WA_SizeGadget, TRUE},
    {WA_DragBar, TRUE},
    {WA_DepthGadget, TRUE},
    {WA_CloseGadget, TRUE},
    {WA_Activate, TRUE},
    {WA_NoCareRefresh, TRUE},
    {WA_GimmeZeroZero, TRUE},
    {WA_SmartRefresh, TRUE},
    {WA_WindowName, (ULONG) "3D-View"},
    {TAG_END}
};

WINDOWDATA drawwindow =
{
    WINDOW_DRAW,
    &NewDrawWindow,
    WINFLAG_TAGWINDOW
};
