/*
 * source/edit/handlemenu.c
 * SpacePigs VectorMania Edit part, menu decoding
 */

#include "other/debug.h"
#include "vector/vector.h"
#include "graphics/graphics.h"
#include "edit/edit.h"
#include "edit/edit_globals.h"

/*** Menyer ***
 * Project
 *	NEW			Radera allt
 *	LOAD ... (L)		Ladda
 *	SAVE ... (S)		Spara
 *	QUIT ... (Q)		Quit
 * EDIT
 *	SELECT 		Vlj punkter
 *		VERTEX			nrmsta punkt
 *		ELEMENT			element med nrmsta punkt
 *		 SELECTED		element med alla valda
 *		 CONNECTED		element med ngon vald
 *		 ALL CONNECTED		allt som hnger ihop med ngon vald
 *		ALL			allt
 *	ERASE  
 *		VERTEX			nrmsta punkt
 *		 SELECTED		valda punkter
 *		ELEMENT			element med nrmsta punkt
 *		 SELECTED		element med alla valda
 *		 CONNECTED		element med ngon vald
 *		 ALL CONNECTED		allt som hnger ihop med ngon vald
 *		ALL			allt
 *	GRAB   
 *		VERTEX			nrmsta punkt
 *		 SELECTED		valda punkter
 *		ELEMENT			element med nrmsta punkt
 *		 SELECTED		element med alla valda
 *		 CONNECTED		element med ngon vald
 *		 ALL CONNECTED		allt som hnger ihop med ngon vald
 *		ALL			allt
 *	COPY   
 *		VERTEX			nrmsta punkt
 *		 SELECTED		valda punkter
 *		ELEMENT			element med nrmsta punkt
 *		 SELECTED		element med alla valda
 *		 CONNECTED		element med ngon vald
 *		 ALL CONNECTED		allt som hnger ihop med ngon vald
 *		ALL			allt
 *	SNAP   		Flytta markren till
 *		NEAREST			nrmsta punk
 *		 SELECTED		nrmsta vald punkt
 *		CENTER			till mitt i fnstren
 *		ZERO			till mitt i vrlden
 *	CENTER 		Flytta fnstercentrum till
 *		NEAREST			nrmsta punk
 *		 SELECTED		nrmsta vald punkt
 *		CURSOR			till markren
 *		ZERO			till mitt i vrlden
 * PREFS
 *	V OK-PROMPTS
 *	V X-WINDOW
 *	V Y-WINDOW
 *	V Z-WINDOW
 *	V Workbench
 *	Colors  ... (O)		Penfrger fr de olika typerna
 *	Palette ... (P)		Paletten
 *
 */

int handlemenu_project(int itemnum, int subnum, int unused, int doubleclick, struct windowdata * windat);
int handlemenu_tool(int itemnum, int subnum, int unused, int doubleclick, struct windowdata * windat);
int handlemenu_grid(int itemnum, int subnum, int unused, int doubleclick, struct windowdata * windat);
int handlemenu_ball(int itemnum, int subnum, int unused, int doubleclick, struct windowdata * windat);
int handlemenu_edit(int itemnum, int subnum, int unused, int doubleclick, struct windowdata * windat);
int handlemenu_select(int itemnum, int subnum, int unused, int doubleclick, struct windowdata * windat);

int handlemenu(int menunum, int itemnum, int subnum, int doubleclick,struct windowdata * windat)
{
    int keepgoing = 1;

    switch (menunum) {
    case MENU_PROJECT:
	keepgoing = handlemenu_project(itemnum, subnum, ~0, doubleclick, windat);
	break;
    case MENU_TOOL:
	keepgoing = handlemenu_tool(itemnum, subnum, ~0, doubleclick, windat);
	break;
    case MENU_GRID:
	keepgoing = handlemenu_grid(itemnum, subnum, ~0, doubleclick, windat);
	break;
    case MENU_BALL:
	keepgoing = handlemenu_ball(itemnum, subnum, ~0, doubleclick, windat);
	break;
    case MENU_EDIT:
	keepgoing = handlemenu_edit(itemnum, subnum, ~0, doubleclick, windat);
	break;
    default:
	saydebug("menu: Unhandled menu:%d\n", menunum);
	break;
    }
    return keepgoing;
}

int handlemenu_project(int itemnum, int subnum, int unused, int doubleclick,struct windowdata * windat)
{
    int keepgoing = 1;

    switch (itemnum) {
    case MENU_PROJECT_SAVE:
	saveeditobj(CURRENTFILENAME);
	break;
    case MENU_PROJECT_SAVEAS:
	saveeditobj(REQUESTFILENAME);
	break;
    case MENU_PROJECT_LOAD:
	loadeditobj(REQUESTFILENAME);
	break;
    case MENU_PROJECT_NEW:
	neweditobj();
	break;
    case MENU_PROJECT_QUIT:
	saydebug("User likes to leave us now.. See ya' later!", 0);
	keepgoing = 0;
	break;
    default:
	saydebug("menu: Unhandled projectmenuitem:%d\n", itemnum);
	break;
    }
    return keepgoing;
}

int handlemenu_tool(int itemnum, int subnum, int unused, int doubleclick,struct windowdata * windat)
{
    int keepgoing = 1;

    switch (itemnum) {
    case MENU_TOOL_ACTION:
    	tool_action(&cursor,doubleclick,windat);
    	break;
    case MENU_TOOL_CANCEL:
	tool_cancel();
	break;
    case MENU_TOOL_NONE:
	settooltype(TOOL_NONE, windat->type);
	break;
    case MENU_TOOL_VERTEX:
	settooltype(TOOL_VERTEX, windat->type);
	break;
    case MENU_TOOL_SURFACE:
	settooltype(TOOL_SURFACE, windat->type);
	break;
    case MENU_TOOL_LINE:
	settooltype(TOOL_LINE, windat->type);
	break;
    case MENU_TOOL_BALLS:
	settooltype(TOOL_BALL, windat->type);
	break;
    case MENU_TOOL_GRAB:
	settooltype(TOOL_GRAB, windat->type);
	break;
    case MENU_TOOL_SQUARE:
	settooltype(TOOL_SQUARE, windat->type);
	break;
    case MENU_TOOL_CUBE:
	settooltype(TOOL_CUBE, windat->type);
	break;
    case MENU_TOOL_BAR1:
    case MENU_TOOL_BAR2:
    	break;
    default:
	saydebug("menu: Unhandled toolmenuitem:%d\n", itemnum);
	settooltype(TOOL_NONE, windat->type);
	break;
    }
    return keepgoing;
}

int handlemenu_grid(int itemnum, int subnum, int unused, int doubleclick,struct windowdata * windat)
{
    int keepgoing = 1;

    setgrid(itemnum + 1);
    return keepgoing;
}

int handlemenu_ball(int itemnum, int subnum, int unused, int doubleclick,struct windowdata * windat)
{
    int keepgoing = 1;

    editballradie=itemnum+1;
    
    return keepgoing;
}

int handlemenu_edit(int itemnum, int subnum, int unused, int doubleclick,struct windowdata * windat)
{
    int keepgoing = 1;

    switch (itemnum) {
    case MENU_EDIT_SELECT:
	keepgoing = handlemenu_select(subnum, ~0, ~0, doubleclick, windat);
	break;
    default:
	saydebug("menu: Unhandled editmenuitem:%d\n", itemnum);
	break;
    }
    return keepgoing;
}

int handlemenu_select(int subnum, int unused1, int unused2, int doubleclick,struct windowdata * windat)
{
    int keepgoing = 1;

    switch (subnum) {
    case MENU_SELECT_VERTEX:
	selectnearcursor(windat->type);
	draweditpoints();
	break;
    default:
	saydebug("menu: Unhandled selectmenusubitem:%d\n", subnum);
	break;
    }
    return keepgoing;
}
