/*
 * source/vector/vector_gammalt.c
 * SpacePigs VectorMania
 */

#include "other/debug.h"
#include "graphics/graphics.h"
#include "vector/vector.h"

/* makesurf(surf,cbase,numcords,cords,type)
 * input:	ELEMENT_SURF,CORD3D cordbase,numcords,cords,DRAW_TYPE
 * output:	ELEMENT_SURF
 * function:	calculate surfacedata and store in ELEMENT_SURF
 */
struct element_surf *
 makesurf(struct element_surf * surf, struct Cord3d * cbase,
	   char numcords, char *cords, enum draw_type type)
{
    int cord;

    if (type == 0)
	type = DT_SURF;

    initelement(&surf->element, type, cbase, numcords, cords);
    if (numcords < 3)
	return NULL;
    for (cord = 0; cord < numcords; cord++) {
	struct element_linecords *border = &surf->border[cord];
	border->cords[0] = cords[cord];
	if (cord + 1 < numcords)
	    border->cords[1] = cords[cord + 1];
	else
	    border->cords[1] = cords[0];
	makeline(&border->line, cbase, border->cords);
    }
    /* Ytans ekvation A*x+B*y+C*z+D=0 */
    {
	double x1, y1, z1, x2, y2, z2, x3, y3, z3;
	struct Cord3d *c1, *c2, *c3;

	c1 = &cbase[cords[0]];
	c2 = &cbase[cords[1]];
	c3 = &cbase[cords[2]];
	x1 = c1->x;
	y1 = c1->y;
	z1 = c1->z;
	x2 = c2->x;
	y2 = c2->y;
	z2 = c2->z;
	x3 = c3->x;
	y3 = c3->y;
	z3 = c3->z;

	surf->c = x1 * (y2 - y3) + x2 * (y3 - y1) + x3 * (y1 - y2);
	if (type == DT_SURFFRONT) {
	    if (surf->c >= 0)
		return NULL;
	    /* Ytan syns inte ! (baksida...) */
	}
	surf->a = y1 * (z2 - z3) + y2 * (z3 - z1) + y3 * (z1 - z2);
	surf->b = z1 * (x2 - x3) + z2 * (x3 - x1) + z3 * (x1 - x2);
	surf->d = -x1 * (y2 * z3 - y3 * z2) - x2 * (y3 * z1 - y1 * z3) - x3 * (y1 * z2 - y2 * z1);
    }
    return surf;
}

/* addsurf(cbase,numcords,cords,type)
 * input: 	CORD3D *cordbase,int numcords,char *cords,DRAW_TYPE type
 * output:	ELEMENT (ELEMENT_SURF)
 * function:	Makes and adds a surface to the drawlist
 */
struct element *
 addsurf(struct Cord3d * cbase, char numcords, char *cords,
	  enum draw_type type)
{
    struct element_surf *surf;
    struct element_surf *doadd;
    surf = malloc(ELEMSURFSIZE(numcords));
    if (surf == NULL)
	ERROR("Can't allocate surface!\n", 20);
    doadd = makesurf(surf, cbase, numcords, cords, type);
    if (doadd) {
	addelement((struct element *) surf, NULL);
    } else {
	free(surf);
	return NULL;
    }
    return &surf->element;
}

/* draw_surf(surf)
 * input:	ELEMENT_SURF
 * function:	Draws a surface on the screen
 */
void draw_surf(struct element * surf)
{
    struct element_surf *face = (struct element_surface *) surf;
    if (face->c <= 0)
	setcolor(GRAY1);
    if (face->c > 0)
	setcolor(GRAY5);
    surface(surf->cbase, surf->numcords, surf->cords);
}

/* overlap_surf_surf(surf1,surf2)
 * input: 	ELEMENT_SURF *surf1,ELEMENT_SURF *surf2
 * output:	int
 * Function:	returns 1 if surf1 overlaps surf2, othervise 0
 */
int overlap_surf_surf(ELEMENT_SURF * surf1, ELEMENT_SURF * surf2)
{
    int stat;
    stat = check_elem_plane((ELEMENT *) surf1, surf2);
    if (stat == 2 || stat == 3)
	return 0;
    stat = check_elem_plane((ELEMENT *) surf2, surf1);
    if (stat == 1 || stat == 3)
	return 0;
    if (!check_border_border(surf1, surf2))
	return 0;
    return 1;
}

/* check_border_border(surf1,surf2)
 * input:	ELEMENT_SURF surf,ELEMENT_SURF surf
 * output:	int
 * function:
 */
int check_border_border(ELEMENT_SURF * surf1, ELEMENT_SURF * surf2)
{
    int num1, num2, pos1, pos2;
    int cross;
    ELEMENT_LINE *line1, *line2;

    num1 = surf1->element.numcords;
    num2 = surf2->element.numcords;
    cross = 0;
    for (pos1 = 0; pos1 < num1; pos1++) {
	line1 = (ELEMENT_LINE *) & surf1->border[pos1];
	for (pos2 = 0; pos2 < num2; pos2++) {
	    line2 = (ELEMENT_LINE *) & surf2->border[pos2];
	    if (check_overlap((ELEMENT *) line1, (ELEMENT *) line2)) {
		if (overlap_line_line(line1, line2)) {
		    return 1;
		}
	    }
	}
    }
    return 0;
}

/* check_elem_plane(surf,plane)
 * input:	ELEMENT_SURF surf,ELEMENT_SURF plane
 * output:	int
 * function:	returns 0 if elem is both behind and infront of plane.. *		returns 1 if all points in elem is infront of plane
 *		returns 2 if all points in elem is behind plane
 *		returns 3 if all points in elem is in plane
 */
int check_elem_plane(ELEMENT * elem, ELEMENT_SURF * plane)
{
    int overlap, coverlap;
    int numcords;
    double a, b, c, d;
    double x, y, z;
    int cordnum;
    struct Cord3d *cbase;
    struct Cord3d *cord;
    char *cords;
    int differ;

    a = plane->a;
    b = plane->b;
    c = plane->c;
    d = plane->d;
    numcords = elem->numcords;
    cbase = elem->cbase;
    cords = elem->cords;

    overlap = 0;
    differ = 0;
    for (cordnum = 0; cordnum < numcords; cordnum++) {
	cord = &cbase[cords[cordnum]];
	if (checkcordinelem(cord, (ELEMENT *) plane))
	    continue;		/* cord in plane... */
	x = cord->x;
	y = cord->y;
	z = cord->z;
	coverlap = x * a + y * b + z * c + d;
	if (coverlap > 0)
	    coverlap = 1;	/* front */
	else if (coverlap < 0)
	    coverlap = -1;	/* behind */
	else if (coverlap == 0)
	    continue;		/* in plane */
	if (overlap == 0)
	    overlap = coverlap;
	else if (coverlap != overlap)
	    differ = 1;
    }
    if (c > 0)
	overlap = -overlap;
    if (differ)
	return 0;
    if (overlap > 0)
	return 1;
    if (overlap < 0)
	return 2;
    if (overlap == 0)
	return 3;
}

void makeline(struct element_line * line, struct Cord3d * cbase, char *cords)
{
    char start = cords[0];
    char end = cords[1];
    struct Cord3d *sc = &cbase[start], *ec = &cbase[end];

    initelement(&line->element, DT_LINE, cbase, 2, cords);

    line->dx = ec->x - sc->x;
    line->dy = ec->y - sc->y;
    line->dz = ec->z - sc->z;
}

struct element *
 addline(struct Cord3d * cbase, char *cords)
{
    struct element_line *line;

    line = malloc(ELEMLINESIZE(1));
    if (line == NULL)
	ERROR("Can't allocate line!\n", 20);
    makeline(line, cbase, cords);
    addelement(&line->element, NULL);

    return &line->element;
}

void draw_line(struct element * elem)
{
    setcolor(WHITE);
    line(elem->cbase, elem->cords[0], elem->cords[1]);
}

/* overlap_line_line(line1,line2)
 * input: 	ELEMENT_LINE line1,line2
 * output:	1 if line1 overlaps line2, 0 if not
 * function:	check if line1 overlaps line2
 */
int overlap_line_line(ELEMENT_LINE * line1, ELEMENT_LINE * line2)
{
    double c1 = 0, c2 = 0, z1, z2;
    if (!cross_line_line(&c1, line1, &c2, line2))
	return 0;
    z1 = c1 * line1->dz;
    z1 += line1->element.cbase[line1->element.cords[0]].z;
    z2 = c2 * line2->dz;
    z2 += line2->element.cbase[line2->element.cords[0]].z;
    if (z2 < z1)
	return 0;
    return 1;
}

/* cross_line_line(*x1,line1,*x2,line2)
 * input:	double *x1,ELEMENT_LINE line1,double *x2,ELEMENT_LINE line2
 * output:	1 if lines cross, 0 if not
 *		x1=line1 cross
 *		x2=line2 cross *
 * function:	find where lines cross
 */
int cross_line_line(double *x1, ELEMENT_LINE * line1, double *x2, ELEMENT_LINE * line2)
{
    double a, b, c, d, e, f, g, h, l, m, temp, div;
    struct Cord3d *c1, *c2, *c3;

    *x1 = 0;
    *x2 = 0;
    c1 = &line1->element.cbase[line1->element.cords[0]];
    c2 = &line2->element.cbase[line2->element.cords[0]];
    c3 = &line1->element.cbase[line1->element.cords[1]];
    if (checkcordinelem(c1, (ELEMENT *) line2))
	return 0;
    if (checkcordinelem(c3, (ELEMENT *) line2))
	return 0;
    a = line1->dx;
    b = line1->dy;
    c = line2->dx;
    d = line2->dy;
    e = c1->x;
    f = c1->y;
    g = c2->x;
    h = c2->y;

    div = b * c - d * a;
    if (div == 0) {
	return 0;
    }
    temp = d * (e - g) + c * (h - f);
    l = temp / div;

    if (l <= 0 || l >= 1) {
	return 0;
    }
    if (c != 0) {
	m = (a * l + e - g) / c;
    } else if (d != 0) {
	m = (b * l + f - h) / d;
    } else {
	saydebug("ERROR: Can't calculate second line cross\n", 0);
	return 0;
    }

    if (m <= 0 || m >= 1) {
	return 0;
    }
    *x1 = l;
    *x2 = m;
    return 1;
}
